Package: betterChromVAR
Title: Improved ChromVAR (Chromatin Variation Across Regions)
Description: 
  A much faster analytical implementation of chromVAR, with additional 
  features, used to infer TF activity from (bulk or single-cell) ATAC-seq data and 
  motif annotations (or binding probabilities). The package also includes the 
  CVnorm normalization method based on the chromVAR logic.
Version: 1.1.9
Authors@R: 
    person("Pierre-Luc", "Germain", email="pierre-luc.germain@hest.ethz.ch", role=c("aut","cre"),
           comment=c(ORCID="0000-0003-3418-4218"))
License: GPL (>= 3)
Encoding: UTF-8
Roxygen: list(markdown = TRUE)
URL: https://github.com/plger/betterChromVAR
BugReports: https://github.com/plger/betterChromVAR/issues
biocViews: Software, ATACSeq, Normalization, Epigenetics, Sequencing
Depends: R (>= 4.1.0), SummarizedExperiment
Imports: BiocParallel, BiocNeighbors, Biostrings, DelayedMatrixStats,
        GenomicRanges, IRanges, Matrix, matrixStats, sparseMatrixStats,
        MatrixGenerics, methods, S4Vectors, stats
Suggests: BiocStyle, knitr, rmarkdown, sessioninfo, testthat
Config/testthat/edition: 3
VignetteBuilder: knitr
Config/roxygen2/version: 8.0.0
git_url: https://git.bioconductor.org/packages/betterChromVAR
git_branch: devel
git_last_commit: 051b316
git_last_commit_date: 2026-08-21
Repository: Bioconductor 3.24
Date/Publication: 2026-08-21
NeedsCompilation: no
Packaged: 2026-08-21 20:36:19 UTC; biocbuild
Author: Pierre-Luc Germain [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-3418-4218>)
Maintainer: Pierre-Luc Germain <pierre-luc.germain@hest.ethz.ch>
