# Generated by roxygen2: do not edit by hand

export(CVnorm)
export(addGCBias)
export(betterChromVAR)
export(computeBackgrounds)
export(computeDeviationsAnalytic)
export(computeDeviationsFromKNN)
export(computeDeviationsWeighted)
export(computeMotifVariability)
export(getBackgroundBins)
export(getBackgroundKNN)
export(getDummyData)
export(getExpectation)
export(normalizeDevsForSize)
export(sampleBackgroundPeaks)
export(shrinkColumnProps)
exportClasses(bcvBackground)
exportMethods("[")
exportMethods(as.list)
exportMethods(show)
importClassesFrom(Matrix,dgCMatrix)
importClassesFrom(Matrix,dgeMatrix)
importClassesFrom(Matrix,sparseMatrix)
importFrom(BiocNeighbors,findKNN)
importFrom(BiocParallel,MulticoreParam)
importFrom(BiocParallel,SerialParam)
importFrom(BiocParallel,bplapply)
importFrom(BiocParallel,bpnworkers)
importFrom(Biostrings,getSeq)
importFrom(Biostrings,letterFrequency)
importFrom(DelayedMatrixStats,rowMeans2)
importFrom(GenomicRanges,GRanges)
importFrom(GenomicRanges,width)
importFrom(IRanges,IRanges)
importFrom(Matrix,Diagonal)
importFrom(Matrix,Matrix)
importFrom(Matrix,cbind2)
importFrom(Matrix,colSums)
importFrom(Matrix,crossprod)
importFrom(Matrix,drop0)
importFrom(Matrix,kronecker)
importFrom(Matrix,rowMeans)
importFrom(Matrix,rowSums)
importFrom(Matrix,sparseMatrix)
importFrom(Matrix,t)
importFrom(Matrix,tcrossprod)
importFrom(MatrixGenerics,rowMeans2)
importFrom(S4Vectors,metadata)
importFrom(SummarizedExperiment,"assay<-")
importFrom(SummarizedExperiment,"mcols<-")
importFrom(SummarizedExperiment,"rowData<-")
importFrom(SummarizedExperiment,"rowRanges<-")
importFrom(SummarizedExperiment,SummarizedExperiment)
importFrom(SummarizedExperiment,assay)
importFrom(SummarizedExperiment,assayNames)
importFrom(SummarizedExperiment,colData)
importFrom(SummarizedExperiment,mcols)
importFrom(SummarizedExperiment,rowData)
importFrom(SummarizedExperiment,rowRanges)
importFrom(matrixStats,colMaxs)
importFrom(matrixStats,colMins)
importFrom(matrixStats,rowQuantiles)
importFrom(matrixStats,rowSds)
importFrom(methods,as)
importFrom(methods,is)
importFrom(methods,new)
importFrom(methods,validObject)
importFrom(sparseMatrixStats,rowVars)
importFrom(stats,cor)
importFrom(stats,cov)
importFrom(stats,dist)
importFrom(stats,dnorm)
importFrom(stats,lm)
importFrom(stats,median)
importFrom(stats,p.adjust)
importFrom(stats,pchisq)
importFrom(stats,poly)
importFrom(stats,qchisq)
importFrom(stats,rnbinom)
importFrom(stats,rnorm)
importFrom(stats,setNames)
importFrom(utils,setTxtProgressBar)
importFrom(utils,txtProgressBar)
importMethodsFrom(methods,show)
