% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{normalizeDevsForSize}
\alias{normalizeDevsForSize}
\title{normalizeDevsForSize}
\usage{
normalizeDevsForSize(dev)
}
\arguments{
\item{dev}{A SummarizedExperiment object as produced by
\code{\link{betterChromVAR}} or \code{\link{computeDeviationsAnalytic}}.}
}
\value{
The \code{dev} object with an additional assay named 'norm'.
}
\description{
Normalizes the z-scores assay of a deviations object to make the scores
comparable across motifs with different number of matches.
}
\examples{
attach(getDummyData())
dev <- betterChromVAR(counts, motifMatches)
dev <- normalizeDevsForSize(dev)
dev
}
