% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{addGCBias}
\alias{addGCBias}
\title{addGCBias}
\usage{
addGCBias(object, genome)
}
\arguments{
\item{object}{An object inheriting RangedSummarizedExperiment or GRanges.}

\item{genome}{A BSgenome object or any other genome object supported by
\code{\link[Biostrings]{getSeq}}.}
}
\value{
\code{object} with the GC content in \code{mcols(object)$bias} (if GRanges)
or \code{rowData(object)$bias}.
}
\description{
Add the \code{bias} column to the object's rowData, containing the regions'
proportion of Gs and Cs.
}
\examples{
# not run:
# se <- addGCBias(se, genome)
}
