% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{getExpectation}
\alias{getExpectation}
\title{getExpectation}
\usage{
getExpectation(counts, grouping = NULL, normalize = TRUE)
}
\arguments{
\item{counts}{A count matrix, or object inheriting SummarizedExperiment with
a 'counts' assay.}

\item{grouping}{An optional vector of length equal to \code{ncol(counts)},
indicating the grouping of the cells. If provided, cells will be averaged
by group before averaging across groups.}

\item{normalize}{Logical; whether to normalize data between averaging (but
after grouping). Default TRUE and highly recommended if providing
\code{grouping}.}
}
\value{
A vector of expectation for each row of \code{counts}
}
\description{
Computes expected counts (a glorified rowMeans)
}
\examples{
attach(getDummyData())
e <- getExpectation(counts)
}
