% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{getDummyData}
\alias{getDummyData}
\title{Dummy data for testing purposes}
\usage{
getDummyData(nRegions = 500, nSamples = 10, nMotifs = 5)
}
\arguments{
\item{nRegions}{Number of regions to generate}

\item{nSamples}{Number of samples to generate}

\item{nMotifs}{Number of motifs to generate}
}
\value{
A list with the slots \code{counts} (a peak counts SummarizedExperiment)
and \code{matches} (a sparse matrix of binary motif matches per peaks)
}
\description{
Dummy data for testing purposes
}
\examples{
out <- getDummyData()
(counts <- out$counts)
matches <- out$motifMatches
}
