% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bcvBackground.R
\name{as.list,bcvBackground-method}
\alias{as.list,bcvBackground-method}
\alias{show,bcvBackground-method}
\alias{[,bcvBackground,ANY,ANY,ANY-method}
\title{Coerce bcvBackground to a list}
\usage{
\S4method{as.list}{bcvBackground}(x)

\S4method{show}{bcvBackground}(object)

\S4method{[}{bcvBackground,ANY,ANY,ANY}(x, i, j, ..., drop = TRUE)
}
\arguments{
\item{x}{A \code{bcvBackground} object.}

\item{object}{A \code{bcvBackground} object.}

\item{i, j}{Indices for subsetting (if j is provided, i is ignored).}

\item{...}{Additional arguments.}

\item{drop}{Logical, whether to drop dimensions.}
}
\value{
A \code{list} containing the slots of the object.

Nothing, prints an overview of the object.

An \code{bcvBackground} object.
}
\description{
Coerce bcvBackground to a list

Show a bcvBackground object

Subsetting a bcvBackground
}
