% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/computeVariability.R
\name{computeMotifVariability}
\alias{computeMotifVariability}
\title{computeMotifVariability}
\usage{
computeMotifVariability(
  z,
  confInt = 0.95,
  n = 100,
  method = c("bonett", "normal", "bootstrap")
)
}
\arguments{
\item{z}{A matrix of z-scores (with motifs as rows), or a
\code{SummaziedExperiment} object with such a matrix in assay \code{z} (as produced
e.g. by \code{\link{computeDeviationsAnalytic}}).}

\item{confInt}{The the confidence interval (a numeric scalar between 0 and
1, default 0.95).}

\item{n}{The number of bootstrap samples. Ignored unless
\code{method="bootstrap"}.}

\item{method}{The method used to compute the confidence interval. 'normal'
computes it analytically, assuming that the z-scores are normally
distributed. 'bonett' (default) adjusts this analytic estimate for
kurtosis (based on Bonett, Computational Statistics & Data Analysis, 2006).
'bootstrap' uses bootstrapping, which does not scale very well.}
}
\value{
A data.frame containing the variability and confidence interval
around it, as well as significance, for each motif. If \code{z} is a
\code{SummarizedExperiment}, the data.frame will be stored in \code{rowData(z)}.
}
\description{
computeMotifVariability
}
\examples{
# we generate random z-scores:
z <- matrix(rnorm(mean=rnorm(10), sd=runif(10, max=2), 200), nrow=10)
var <- computeMotifVariability
}
