% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bcvBackground.R
\docType{class}
\name{bcvBackground-class}
\alias{bcvBackground-class}
\title{Bin and background data for betterChromVAR (for internal use)}
\description{
Bin and background data for betterChromVAR (for internal use)
}
